;; Copyright 2007, 2008, 2009, 2010, 2011, 2016 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; set this to `t' for some diagnostic messages
(defvar my-test-verbose nil)


(add-to-list 'load-path (expand-file-name ".."))

(unless (fboundp 'file-expand-wildcards)
  (load (expand-file-name "file-expand-wildcards.el")))

(defconst my-gzip-available-p
  (condition-case nil
      (equal 0 (call-process "gzip"
                             nil  ;; stdin
                             nil  ;; stdout
                             nil  ;; display
                             "--version"))
    (error nil))
  "Non-nil if the \"gzip\" program is available.")

(defun my-browse-url-error (url &rest args)
  "For use in `browse-url-browser-function'."
  (error "My browse-url throwing an error"))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(gtk-lookup-symbol))

    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'gtk-lookup)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    ;; gtk-lookup-completing-help-get

    (require 'gtk-look) ;; before let-binding
    (let ((gtk-lookup-cache '(("foo" "Foo.html#foo" "/some/dir/" nil)
                              ("bar" "Bar.html#bar" "/another/dir/" function))))
      (mapcar (lambda (elem)
                (let* ((str  (nth 0 elem))
                       (want (nth 1 elem))
                       (got  (gtk-lookup-completing-help-get str)))
                  (unless (equal got want)
                    (error "gtk-lookup-completing-help-get %S got %S want %S"
                           str got want))
                  (setq count (1+ count))))

              '(("foo" "in Foo.html")
                ("bar" "function, in Bar.html")
                ("unknown" nil))))

    ;;-------------------------------------------------------------------------
    ;; gtk-lookup--with-auto-compression

    (cond ((not my-gzip-available-p)
           (message "skip gtk-lookup--with-auto-compression due to no gzip program"))
          ((not (fboundp 'gtk-lookup--with-auto-compression))
           (message "skip gtk-lookup--with-auto-compression test due to no macro after byte compile"))

          (t
           (let* ((plainfile "t-gtk-look.tmp")
                  (gzfile    (concat plainfile ".gz")))
             (unwind-protect
                 (progn
                   (with-temp-buffer
                     (insert "hello world\n")
                     (write-file plainfile))
                   (call-process "gzip" nil nil nil plainfile)

                   (gtk-lookup--with-auto-compression
                    (find-file gzfile)
                    (unless (equal "hello world\n" (buffer-string))
                      (error "gtk-lookup--with-auto-compression didn't work"))
                    (kill-buffer nil) ;; gzfile
                    (setq count (1+ count))))

               (condition-case nil (delete-file plainfile) (error))
               (condition-case nil (delete-file gzfile)    (error))))))

    ;;-------------------------------------------------------------------------
    ;; gtk-lookup-canonicalize-symbol

    ;; pretend no indexes, so the tests don't vary with what you've got
    ;; installed
    (require 'gtk-look) ;; before let-binding
    (let ((gtk-lookup-cache
           '(("gnome_canvas_new")
             ("glade_xml_new")
             ("gtk_label_new")
             ("gdk_event_new")
             ("gtk_tree_store_new")
             ("gtk_ui_manager_new")
             ("gtk_vscrollbar_new")
             ("GtkMessageDialog")
             ("GtkMessageDialog")
             ("gtk_init")
             ("gtk_main_quit")
             ("gtk_check_version")
             ("GTK_CHECK_VERSION")
             ("GObject")
             ("GObjectClass")
             ("DeclaSee")
             ("GTypeInstance")
             ("G_TYPE_GTYPE")
             ("G_TYPE_BOOLEAN")
             ("gdk_draw_drawable")
             ("G_PRIORITY_LOW")
             ("GTK_PRIORITY_RESIZE")
             ("GDK_PRIORITY_EVENTS")
             ("GDK_SELECTION_PRIMARY")

             ("GTK_STOCK_YES")
             ("GTK_STOCK_ZOOM_100")
             ("GTK_STOCK_QUIT")
             ("gtk_quit")

             ("myaddon-hello-world")
             ("MYADDON_STOCK_HELLO_WORLD")
             )))

      (mapcar (lambda (elem)
                (let* ((str  (nth 0 elem))
                       (want (nth 1 elem))
                       (gtk-lookup-cache (if (>= (length elem) 3)
                                             (nth 2 elem)
                                           gtk-lookup-cache))
                       (got  (gtk-lookup-canonicalize-symbol str)))

                  (if my-test-verbose
                      (message "str: %s" str))
                  (unless (string-equal want got)
                    (error "str %S got %S want %S, with cache %S"
                           str got want gtk-lookup-cache))
                  (setq count (1+ count))))

              '(
                ("myaddon-some-thing"  "myaddon_some_thing")
                ("gtk-quit"     "gtk_quit")
                ("gtk-yes"      "GTK_STOCK_YES")
                ("gtk-zoom-100" "GTK_STOCK_ZOOM_100")
                ("myaddon-hello-world" "MYADDON_STOCK_HELLO_WORLD")

                ("Gnome2::Canvas->new"   "gnome_canvas_new")
                ("Gtk2->foo->bar"        "bar")
                ("MyNewClass->signal_add_emission_hook" "signal_add_emission_hook")
                ("Gtk2->foo" "foo")
                ("Gtk2->foo" "gtk_foo" (("gtk_foo" . nil)))

                ("Gtk2::Gdk::Cairo::Context->create" "gdk_cairo_create"
                 (("gdk_cairo_create" . nil)))

                ("Glib::Param::Double" "GParamSpecDouble")
                ("Glib::Param::Int" "GParamSpecInt")

                ("Gtk2::GladeXML->new"   "glade_xml_new")
                ("gtk-label-new"         "gtk_label_new")
                ("Gtk2::Gdk::Event->new" "gdk_event_new")
                ("Gtk2::TreeStore->new"  "gtk_tree_store_new")
                ("Gtk2::UIManager->new"  "gtk_ui_manager_new")
                ("Gtk2::VScrollbar->new" "gtk_vscrollbar_new")
                ("Gtk2::MessageDialog"   "GtkMessageDialog")
                ("Gtk2::MessageDialog::" "GtkMessageDialog")
                ("Gtk2->init"            "gtk_init")
                ("Gtk2->main_quit"       "gtk_main_quit")
                ("Gtk2->check_version"   "gtk_check_version")
                ("Gtk2->CHECK_VERSION"   "GTK_CHECK_VERSION")
                ("<gobject>"             "gobject")
                ("<gtype-instance>"      "gtypeInstance")
                ("gtype:gtype"           "G_TYPE_gtype")

                ;; FIXME: this one depends on gboolean existing in the
                ;; indexes ...
                ;; ("gtype:gboolean"        "G_TYPE_boolean")

                ("draw_drawable"         "draw_drawable")
                ("draw_draw"             "draw_draw")
                ("Glib::G_PRIORITY_LOW"  "G_PRIORITY_LOW")
                ("Gtk2::GTK_PRIORITY_RESIZE" "GTK_PRIORITY_RESIZE")
                ("Gtk2::GDK_PRIORITY_EVENTS" "GDK_PRIORITY_EVENTS")
                ("Gtk2::Gdk->SELECTION_PRIMARY" "GDK_SELECTION_PRIMARY")

                ;; Class struct fallback
                ("GObjectClass"  "GObjectClass")
                ("DeclaSeeClass" "DeclaSee"))))


    ;;-------------------------------------------------------------------------
    ;; gtk-lookup-string-suffix-ci-p

    (mapcar (lambda (elem)
              (let* ((suff (car elem))
                     (str  (car (cdr elem)))
                     (want (car (cdr (cdr elem))))
                     (got  (gtk-lookup-string-suffix-ci-p suff str)))
                (unless (equal want got)
                  (error "suff %s str %s got %s want %s\n" suff str got want))
                (setq count (1+ count))))

            '(("new" "label_new" t)
              ("new" "NEW" t)
              ("longsuff" "str" nil)
              ("longsuff" "" nil)))


    ;;-------------------------------------------------------------------------
    ;; gtk-lookup-cache

    (gtk-lookup-reset)
    (gtk-lookup-cache-init)
    (mapcar (lambda (elem)
              ;; (NAME LINK DIR TYPE)
              (let* ((name (nth 0 elem))
                     (link (nth 1 elem))
                     (dir  (nth 2 elem))
                     (type (nth 3 elem)))
                (when (string-match "\\s-" name)
                  ;; ignore fishy "method" in gdbus
                  (unless (equal "The Poke() method" name)
                    (error "cache entry \"name\" has whitespace: %S\n%S"
                           name elem)))
                (unless (string-match "\\`[a-zA-Z_]" name)
                  (error "cache entry \"name\" non alpha start: %S\n%S"
                         name elem))
                (when (string-match "\\s-" link)
                  (error "cache entry \"link\" has whitespace: %S\n%S"
                         link elem))
                (when (string-match "\\s-" dir)
                  (error "cache entry \"dir\" has whitespace: %S\n%S"
                         dir elem))
                (unless (symbolp type)
                  (error "cache entry \"type\" not a symbol: %S\n%S"
                         type elem))))
            gtk-lookup-cache)
    (message "gtk-lookup-cache size %S" (length gtk-lookup-cache))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; gtk-lookup-browse-url-other-window

    ;; check current buffer unchanged on error
    (require 'browse-url)
    (with-temp-buffer
      (let* ((want-buffer (current-buffer))
             (want-msg    "My browse-url throwing an error")
             (browse-url-browser-function (lambda (url &optional new-session)
                                            (error "%s" want-msg)))
             got-buffer
             got-msg)
        (condition-case err
            (gtk-lookup-browse-url-other-window "file:///tmp")
          (error
           (setq got-msg (error-message-string err))))
        (setq got-buffer (current-buffer))

        (unless (equal got-msg want-msg)
          (error "browse-url error message got %S want %S"
                 got-msg want-msg))
        (unless (equal got-buffer want-buffer)
          (error "browse-url error buffer got %S want %S"
                 got-buffer want-buffer))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (let ((prop '(gtk-lookup-symbol bounds-of-thing-at-point)))
      (unless (apply 'get prop)
        (error "oops, didn't find thing-at-point property"))

      (unload-feature 'gtk-look t)

      (if (featurep 'xemacs)
          (message "skip, xemacs21 unload-feature doesn't purge hook variables")
        (when (memq 'gtk-lookup-display-initial-completion-list
                    minibuffer-setup-hook)
          (error "unload-feature leftover minibuffer-setup-hook %S"
                 minibuffer-setup-hook))
        (setq count (1+ count)))

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)
      (if (not my-unload-feature-function-p)
          (message "skip, no unload-feature function")

        (let ((func (apply 'get prop)))
          (when func
            (error "unload-feature leftover %S property: %S" prop func)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))
