;; Copyright 2013, 2016, 2017 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

;;; Commentary:

;; Create an elpa archive of my .el files.
;; .asc sigs are copied to .sig in the archive.
;; File modtimes are not preserved by package-upload-file, so copy the modtimes
;; explicitly.
;; archive-contents file is signed by elpa-upload.sh.

;;; Code:

(require 'package-x)
(setq package-archive-upload-base "~/tux/data/elpa/packages/")

(defun my-numbered-filename (orig-filename)
"Return the numbered filename for Lisp package ORIG-FILENAME.
ORIG-FILENAME is like \"foo.el\".
Return is like \"~/tux/data/elpa/packages/foo-123.el\".
The version number is taken from files already in elpa/packages/."

  (setq orig-filename (file-name-sans-extension orig-filename))
  (let ((lst (directory-files package-archive-upload-base
                              nil ;; not full
                              (concat "\\`"
                                      (regexp-quote orig-filename)
                                      "-[0-9]+\\.el\\'"))))
    (unless (= (length lst) 1)
      (error "Oops, numbered filename not found for %s" orig-filename))
    (expand-file-name (car lst) package-archive-upload-base)))

(defun my-readme-filename (orig-filename)
  "Return the .readme filename for Lisp package ORIG-FILENAME.
ORIG-FILENAME is like \"foo.el\".
Return is like \"~/tux/data/elpa/packages/foo-readme.txt\".
This .txt file must exist already."
  (setq orig-filename (file-name-sans-extension orig-filename))
  (let ((lst (directory-files package-archive-upload-base
                              nil ;; not full
                              (concat "\\`"
                                      (regexp-quote orig-filename)
                                      "-readme+\\.txt\\'"))))
    (unless (= (length lst) 1)
      (error "Oops, readme filename not found for %s" orig-filename))
    (expand-file-name (car lst) package-archive-upload-base)))

(defun my-sig-filename (orig-filename)
  "Return the .sig filename for Lisp package ORIG-FILENAME.
ORIG-FILENAME is like \"foo.el\".
Return is like \"~/tux/data/elpa/packages/foo.el.sig\"."
  (concat (my-numbered-filename orig-filename) ".sig"))

(defun my-copy-modtime (from-filename to-filename)
  "Copy the modtime of FROM-FILENAME to TO-FILENAME.
Both files should exist."
  (let ((attrs (file-attributes from-filename)))
    (unless (set-file-times to-filename (nth 5 attrs))
      (error "Oops, cannot set-file-times of %s" to-filename))))


(ignore-errors (delete-directory package-archive-upload-base t))
(make-directory package-archive-upload-base t)

(setq default-directory "~/tux/data")

;; reverse so archive-contents is sorted alphabetically
;;
(dolist (filename (reverse (append (file-expand-wildcards "*.el" t)
                                   (file-expand-wildcards "pari/*.el" t))))
  (setq default-directory (file-name-directory filename))
  (setq filename (file-name-nondirectory filename))

  (message "package-upload file %S from %S" filename default-directory)
  (package-upload-file filename)

  (my-copy-modtime filename (my-numbered-filename filename))
  (my-copy-modtime filename (my-readme-filename filename))

  (copy-file (concat filename ".asc")
             (my-sig-filename filename))
  (my-copy-modtime (concat filename ".asc")
                   (my-sig-filename filename)))

;; doesn't work properly run from within emacs -batch
;;
;; (with-temp-buffer
;;   (let ((default-directory package-archive-upload-base))
;;     (if (getenv "DISPLAY")
;;         (call-process "xterm" nil t nil
;;                       "-e"
;;                       "if [ -e ~/bin/my-gpg-agent-daemon ]; then
;;                          eval `my-gpg-agent-daemon`
;;                          echo gpg-agent $GPG_AGENT_INFO
;;                        fi
;;                        gpg -ab -o archive-contents.sig archive-contents")
;;       (call-process "sh" nil t nil
;;                     "-c"
;;                     "if [ -e ~/bin/my-gpg-agent-daemon ]; then
;;                        eval `my-gpg-agent-daemon`
;;                        echo gpg-agent $GPG_AGENT_INFO
;;                      fi
;;                      gpg -ab -o archive-contents.sig archive-contents")))
;;   (message "%s" (buffer-string)))
;; 
;; (with-temp-buffer
;;   (let ((default-directory package-archive-upload-base)
;;         (status (call-process "gpg" nil t nil
;;                               "--verify" "archive-contents.sig" "archive-contents")))
;;     (message "%s" (buffer-string))
;;     (unless (equal 0 status) (error ""))))
;; 
;; (with-temp-buffer
;;   (let ((default-directory package-archive-upload-base)
;;         (status (call-process "ls" nil t nil
;;                               "-l" "archive-contents" "archive-contents.sig")))
;;     (message "%s" (buffer-string))
;;     (unless (equal 0 status) (error "")))
